Base.@kwdef struct ModalDecayHighest <: AbstractTCI
  mesh::Mesh
  threshold_min::Float64
  threshold_max::Float64
  # buffers
  uh_mod::Vector{Float64}
  uh_mod_high::Vector{Float64}
end


function ModalDecayHighest(mesh, threshold_min, threshold_max)
  L       = layout(mesh)
  Npts, K = L
  # storage for modal coefficients
  uh_mod = zeros(Float64, Npts)
  uh_mod_high = zeros(Float64, Npts)
  return ModalDecayHighest(mesh, threshold_min, threshold_max, uh_mod, uh_mod_high)
end


function compute_indicator!(
    flag, # mutated outputs
    u,    # inputs,
    tci::ModalDecayHighest)

  @unpack mesh, threshold_min, threshold_max,
          uh_mod, uh_mod_high = tci
  @unpack invjac, dl, element = mesh
  @unpack D, invV, V          = mesh.element

  L       = layout(mesh)
  Npts, K = L
  N       = Npts - 1
  umat    = reshape(u, L)

  for k in 1:K

    cell_u = view(umat, :, k)
    mul!(uh_mod, invV, cell_u)
    uh_mod_high        .= V[:,end] .* uh_mod[end]
    norm2_uh_mod_high   = dg1d.inner_product(uh_mod_high, uh_mod_high, element) / invjac[k]
    norm2_uh_mod        = dg1d.inner_product(uh_mod, uh_mod, element) / invjac[k]

    S = abs(norm2_uh_mod_high/norm2_uh_mod)
    S = abs(norm2_uh_mod) < 1e-12 ? 1e-20 : S
    S = abs(S) == 0 ? 1e-20 : S

    (isinf(S) || isnan(S)) && error("Encountered Inf or NaN in estimation of highest mode energy!")

    # sensor
    tau = log10(S)

    if threshold_max < tau
      flag[k] = 0.0
    elseif threshold_min <= tau <= threshold_max
      flag[k] = 1.0 - (tau - threshold_min) / (threshold_max - threshold_min)
    else
      flag[k] = 1.0
    end
  end

  return
end
