module dg1d

using Reexport
import Base: Filesystem, Threads
using Glob
using InteractiveUtils
using Jacobi
@reexport using HDF5
using LinearAlgebra
import MacroTools
using OrderedCollections
using Plots
@reexport import Parameters: @unpack, @with_kw
using PrettyTables
using Printf
using ProgressMeter
using REPL.TerminalMenus
import Roots
using StaticArrays
using StructArrays
using TimerOutputs
@reexport using TOML
@reexport using ToggleableAsserts

#######################################################################
#                                utils                                #
#######################################################################

include("sandbox.jl")

export # parameter handling
       query, splitallext, trim_keys, match_keys, prepend_keys,
       # type utils
       concretetypes,
       # numeric utilities
       periodize, periodic_index, # deprecated
       estimate_convergence_rate,
       # parameter files and file system operations
       mk_outputdir, dirname_path, flatten, get_parameter_file, get_parameter_file_index,
       select_parameter_files, gather_output_directories, gather_parameter_files,
       match_properties,
       find_duplicated_parameter_files,
       run_projects, analyze_projects,
       TODO, @TODO, @stringify_names,
       make_nan_separated_storage, indent
include("utils.jl")

export Output, read_variable, read_variable!, read_mesh
include("output.jl")

#######################################################################
#                            Numeric utils                            #
#######################################################################

export find_root
include("rootfinders.jl")
include("fd.jl")
include("lagrange.jl")
include("glgl.jl")
include("lgl.jl")
include("evolve.jl")

#######################################################################
#                             Components                              #
#######################################################################

export SpectralElement
include("spectralelement.jl")

export Mesh, MeshInterpolator, layout, n_points, n_cells,
       grid_average, cellwise_average, cellwise_inner_product, broken_inner_product,
       locate_point, find_cell_index
include("mesh.jl")

export AbstractEquation
include("equations.jl")

export Variables, n_variables, names
include("variablesdict.jl")
# choose between variables interface implementation
# const Variables = _VariablesArray
const Variables = _VariablesDict

export Cache, register_variables!,
       get_global_variables, get_dynamic_variables, get_cell_variables,
       get_bdry_variables, get_rhs_variables, get_static_variables,
       get_variable, wrap_dynamic_variables!, wrap_rhs_variables!,
       broadcast_volume!, variablenames, variablegroups
include("cache.jl")

export CallbackSet, CallbackTiming, FunctionCallback, SaveCallback, PlotCallback,
       TimeAlignedSaveCallback
include("callbacks.jl")

export AbstractRiemannSolver, ApproxRiemannSolver,
       lax_friedrich_flux, avg_lax_friedrich_flux,
       central_flux, mean_flux, broadcast_faces!
include("riemannsolver.jl")

export DirichletBC, OutflowBC, InflowBC, PeriodicBC, BoundaryConditions, isperiodic,
       broadcast_boundaryconditions!
include("bdryconditions.jl")

export @with_signature, @accepts, @returns,
       has_signature, accepts, returns, signature, state_indices
include("with_signature.jl")

export compute_rhs_weak_form!, compute_rhs_strong_form!
include("dg_rhs.jl")

export Maybe, Environment
include("types.jl")

#######################################################################
#                              Projects                               #
#######################################################################

include("projects.jl")

export parameters, @parameters
include("parameters.jl")

export main
include("main.jl")

include("TCI/TCI.jl")
@reexport using .TCI
include("HRSC/HRSC.jl")
@reexport using .HRSC
include("EquationOfState/EquationOfState.jl")
@reexport using .EquationOfState
include("ScalarEq/ScalarEq.jl")
include("EulerEq/EulerEq.jl")
include("SRHD/SRHD.jl")
include("GRHD/GRHD.jl")


end # module
